@@ -639,6 +639,44 @@ cdma_device_info_free (gpointer data)
 	g_free (info);
 }
 
+static void
+notify_user_of_cdma_reg_change (CdmaDeviceInfo *info)
+{
+	guint32 mb_state = cdma_state_to_mb_state (info);
+
+	if (mb_state == MB_STATE_HOME) {
+		applet_do_notify_with_pref (info->applet,
+		                            _("CDMA network."),
+		                            _("You are now registered on the home network."),
+		                            "nm-signal-100",
+		                            PREF_DISABLE_CONNECTED_NOTIFICATIONS);
+	} else if (mb_state == MB_STATE_ROAMING) {
+		applet_do_notify_with_pref (info->applet,
+		                            _("CDMA network."),
+		                            _("You are now registered on a roaming network."),
+		                            "nm-signal-100",
+		                            PREF_DISABLE_CONNECTED_NOTIFICATIONS);
+	}
+}
+
+static void
+update_registration_state (CdmaDeviceInfo *info,
+                           guint32 new_cdma1x_state,
+                           guint32 new_evdo_state)
+{
+	guint32 old_mb_state = cdma_state_to_mb_state (info);
+
+	if (   (info->cdma1x_state != new_cdma1x_state)
+	    || (info->evdo_state != new_evdo_state)) {
+		info->cdma1x_state = new_cdma1x_state;
+		info->evdo_state = new_evdo_state;
+	}
+
+	/* Use the composite state to notify of home/roaming changes */
+	if (cdma_state_to_mb_state (info) != old_mb_state)
+		notify_user_of_cdma_reg_change (info);
+}
+
 static void
 reg_state_reply (DBusGProxy *proxy, DBusGProxyCall *call, gpointer user_data)
 {
@@ -650,8 +688,7 @@ reg_state_reply (DBusGProxy *proxy, DBusGProxyCall *call, gpointer user_data)
 	                           G_TYPE_UINT, &cdma1x_state,
 	                           G_TYPE_UINT, &evdo_state,
 	                           G_TYPE_INVALID)) {
-		info->cdma1x_state = cdma1x_state;
-		info->evdo_state = evdo_state;
+		update_registration_state (info, cdma1x_state, evdo_state);
 		applet_schedule_update_icon (info->applet);
 	}
 
@@ -683,7 +720,7 @@ find_provider_for_sid (GHashTable *table, guint32 sid)
 {
 	GHashTableIter iter;
 	gpointer value;
-	GSList *miter, *piter, *siter;
+	GSList *piter, *siter;
 	char *name = NULL;
 
 	if (sid == 0)
@@ -698,16 +735,11 @@ find_provider_for_sid (GHashTable *table, guint32 sid)
 		for (piter = providers; piter && !name; piter = g_slist_next (piter)) {
 			NmnMobileProvider *provider = piter->data;
 
-			/* Search through each provider's access methods */
-			for (miter = provider->methods; miter && !name; miter = g_slist_next (miter)) {
-				NmnMobileAccessMethod *method = miter->data;
-
-				/* Search through CDMA SID list */
-				for (siter = method->cdma_sid; siter; siter = g_slist_next (siter)) {
-					if (GPOINTER_TO_UINT (siter->data) == sid) {
-						name = g_strdup (provider->name);
-						break;
-					}
+			/* Search through CDMA SID list */
+			for (siter = provider->cdma_sid; siter; siter = g_slist_next (siter)) {
+				if (GPOINTER_TO_UINT (siter->data) == sid) {
+					name = g_strdup (provider->name);
+					break;
 				}
 			}
 		}
@@ -844,10 +876,8 @@ reg_state_changed_cb (DBusGProxy *proxy,
 {
 	CdmaDeviceInfo *info = user_data;
 
-	info->cdma1x_state = cdma1x_state;
-	info->evdo_state = evdo_state;
+	update_registration_state (info, cdma1x_state, evdo_state);
 	info->skip_reg_poll = TRUE;
-
 	applet_schedule_update_icon (info->applet);
 }
 
